/* Sylpheed Pro full-text search plugin
 * Copyright (C) 2007-2009 Sylpheed Development Team
 * Copyright (C) 2009 SRA OSS, Inc. Japan
 */

#include <string.h>
#include <time.h>

#include "sylph/sylmain.h"
#include "sylph/prefs.h"
#include "sylph/utils.h"

#include "common.h"


static AppConfig tmp_config;

static PrefParam param[] = {
	{"dbname", NULL, &tmp_config.dbname, P_STRING},
	{"hostname", NULL, &tmp_config.hostname, P_STRING},
	{"port", "0", &tmp_config.port, P_USHORT},
	{"user", NULL, &tmp_config.user, P_STRING},
	{"pass", NULL, &tmp_config.pass, P_STRING},
	{"dbpath", NULL, &tmp_config.dbpath, P_STRING},
	{"res_limit", "0", &tmp_config.res_limit, P_INT},
	{"auto_update", "TRUE", &tmp_config.auto_update, P_BOOL},
	{"enable_searcher", "TRUE", &tmp_config.enable_searcher, P_BOOL},
	{"exclude_list", "junk,spam,trash", &tmp_config.exclude_list, P_STRING},
	{NULL, NULL, NULL, P_OTHER}
};

void update_exclude_list(AppConfig *config)
{
	gint i;

	if (config->exclude_list_v)
		g_strfreev(config->exclude_list_v);
	config->exclude_list_v = g_strsplit(config->exclude_list, ",", -1);

	for (i = 0; config->exclude_list_v[i] != NULL; i++)
		g_strstrip(config->exclude_list_v[i]);
}

gint read_config(AppConfig *config)
{
	gchar *path;

	g_return_val_if_fail(config != NULL, -1);

	memset(&tmp_config, 0, sizeof(tmp_config));

	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "searcherrc", NULL);
	if (!is_file_exist(path)) {
		g_free(path);
		return -1;
	}
	prefs_read_config(param, "Searcher", path, NULL);
	g_free(path);

	*config = tmp_config;
	memset(&tmp_config, 0, sizeof(tmp_config));

	update_exclude_list(config);

	return 0;
}

gint write_config(AppConfig *config)
{
	g_return_val_if_fail(config != NULL, -1);

	tmp_config = *config;
	prefs_write_config(param, "Searcher", "searcherrc");
	memset(&tmp_config, 0, sizeof(tmp_config));

	return 0;
}

gchar *read_license_key()
{
	FILE *fp;
	gchar *path;
	gchar buf[256];

	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "license-key.ini", NULL);
	if (!is_file_exist(path)) {
		g_free(path);
		return NULL;
	}

	if ((fp = g_fopen(path, "rb")) == NULL) {
		FILE_OP_ERROR(path, "fopen");
		g_free(path);
		return NULL;
	}

	if (fgets(buf, sizeof(buf), fp) == NULL) {
		g_warning("reading license key file failed: %s", path);
		fclose(fp);
		g_free(path);
		return NULL;
	}
	g_strstrip(buf);
	fclose(fp);
	g_free(path);

	return g_strdup(buf);
}

gint write_license_key(const gchar *key)
{
	FILE *fp;
	gchar *path;
	gint ret = 0;

	g_return_val_if_fail(key != NULL, -1);

	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "license-key.ini", NULL);
	if ((fp = g_fopen(path, "wb")) == NULL) {
		FILE_OP_ERROR(path, "fopen");
		g_free(path);
		return -1;
	}

	if (fputs(key, fp) == EOF) {
		perror("fputs");
		ret = -1;
	}
	fclose(fp);
	g_free(path);

	return ret;
}
