/* Sylpheed Pro full-text search plugin
 * Copyright (C) 2007-2009 Sylpheed Development Team
 * Copyright (C) 2009 SRA OSS, Inc. Japan
 */

#include "config.h"

#include <glib.h>
#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>
#include <stdlib.h>

#include "common.h"
#include "plugin.h"
#include "searcher-config.h"
#include "sylph-searcher.h"
#include "dbimport.h"

static void license_clicked(GtkButton *button, gpointer data);

#define ATTACH_LABEL_ENTRY(str, entry) \
{ \
	hbox = gtk_hbox_new(FALSE, 0); \
	label = gtk_label_new(str); \
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0); \
	gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, row, row + 1, \
			 GTK_FILL, 0, 0, 0); \
	hbox = gtk_hbox_new(FALSE, 0); \
	entry = gtk_entry_new(); \
	gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE); \
	gtk_widget_set_sensitive(entry, FALSE); \
	gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0); \
	gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, row, row + 1, \
			 GTK_FILL, 0, 0, 0); \
	++row; \
}

gint searcher_config(AppConfig *config, GtkWindow *parent)
{
	GtkWidget *dialog;
	GtkWidget *vbox;
	GtkWidget *enable_chkbtn;
	GtkWidget *frame;
	GtkWidget *table;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *dbpath_label;
	GtkWidget *dbname_entry;
	GtkWidget *host_entry;
	GtkWidget *port_entry;
	GtkWidget *user_entry;
	GtkWidget *pass_entry;
	GtkWidget *limit_entry;
	GtkWidget *vbox2;
	GtkWidget *auto_update_chkbtn;
	GtkWidget *exclude_entry;
	GtkWidget *license_btn;
	GtkTooltips *tip;
	gint row = 0;
	gchar buf[64];
	gint result;

	g_return_val_if_fail(config != NULL, FALSE);

	dialog = gtk_dialog_new_with_buttons
		(_("Full-text search settings"), NULL, 0,
		 GTK_STOCK_OK, GTK_RESPONSE_OK,
		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		 NULL);
	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, FALSE);
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

	vbox = gtk_vbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox,
			   FALSE, FALSE, 0);

	hbox = gtk_hbox_new(FALSE, 4);
	enable_chkbtn = gtk_check_button_new_with_label
		(_("Enable full-text search feature"));
	gtk_box_pack_start(GTK_BOX(hbox), enable_chkbtn, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	tip = gtk_tooltips_new();
	gtk_tooltips_set_tip(tip, enable_chkbtn, _("Changes of this setting require restart"), NULL);

	frame = gtk_frame_new(_("Database"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	vbox2 = gtk_vbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 8);
	gtk_container_add(GTK_CONTAINER(frame), vbox2);

	table = gtk_table_new(6, 2, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
	gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 0);

	ATTACH_LABEL_ENTRY(_("Database name:"), dbname_entry);
	ATTACH_LABEL_ENTRY(_("Hostname:"), host_entry);
	ATTACH_LABEL_ENTRY(_("Port:"), port_entry);
	ATTACH_LABEL_ENTRY(_("Username:"), user_entry);
	ATTACH_LABEL_ENTRY(_("Password:"), pass_entry);
	gtk_entry_set_visibility(GTK_ENTRY(pass_entry), FALSE);

	hbox = gtk_hbox_new(FALSE, 4);
	label = gtk_label_new(_("Location:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	dbpath_label = gtk_label_new("");
	gtk_label_set_selectable(GTK_LABEL(dbpath_label), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), dbpath_label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);

	frame = gtk_frame_new(_("Search index"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	vbox2 = gtk_vbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 8);
	gtk_container_add(GTK_CONTAINER(frame), vbox2);

	hbox = gtk_hbox_new(FALSE, 4);
	auto_update_chkbtn = gtk_check_button_new_with_label
		(_("Enable auto update of search index"));
	gtk_box_pack_start(GTK_BOX(hbox), auto_update_chkbtn, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);

	hbox = gtk_hbox_new(FALSE, 4);
	label = gtk_label_new(_("Exclude folders which include the following strings (comma-separated):"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
	hbox = gtk_hbox_new(FALSE, 4);
	exclude_entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), exclude_entry, TRUE, TRUE, 0);
	label = gtk_label_new(_("(Ex: spam,junk)"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);

	hbox = gtk_hbox_new(FALSE, 4);
	label = gtk_label_new(_("Maximum display number of search result:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	limit_entry = gtk_entry_new();
	gtk_widget_set_size_request(limit_entry, 80, -1);
	gtk_box_pack_start(GTK_BOX(hbox), limit_entry, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	license_btn = gtk_button_new_with_mnemonic(_("_License information..."));
	gtk_box_pack_start(GTK_BOX(vbox), license_btn, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(license_btn), "clicked",
			 G_CALLBACK(license_clicked), dialog);

	/* Apply config to UI */

#define SET_TEXT(entry, text) \
	if (text) { \
		gtk_entry_set_text(GTK_ENTRY(entry), text); \
	}

	if (config->dbpath)
		gtk_label_set_text(GTK_LABEL(dbpath_label), config->dbpath);
	SET_TEXT(dbname_entry, config->dbname);
	SET_TEXT(host_entry, config->hostname);
	if (config->port > 0) {
		g_snprintf(buf, sizeof(buf), "%u", config->port);
		gtk_entry_set_text(GTK_ENTRY(port_entry), buf);
	}
	SET_TEXT(user_entry, config->user);
	SET_TEXT(pass_entry, config->pass);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(auto_update_chkbtn),
				     config->auto_update);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_chkbtn),
				     config->enable_searcher);
	SET_TEXT(exclude_entry, config->exclude_list);

	if (config->res_limit > 0) {
		g_snprintf(buf, sizeof(buf), "%d", config->res_limit);
		gtk_entry_set_text(GTK_ENTRY(limit_entry), buf);
	}

#undef SET_TEXT

	gtk_widget_show_all(dialog);
	if (parent)
		gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(parent));
	else
		syl_plugin_manage_window_set_transient(GTK_WINDOW(dialog));

	syl_plugin_inc_lock();
	result = gtk_dialog_run(GTK_DIALOG(dialog));

	/* Apply UI to config */

#define UPDATE_MEMBER(member, entry) \
{ \
	g_free(config->member); \
	config->member = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1); \
}

	if (result == GTK_RESPONSE_OK) {
		UPDATE_MEMBER(dbname, dbname_entry);
		UPDATE_MEMBER(hostname, host_entry);
		config->port = atoi(gtk_entry_get_text(GTK_ENTRY(port_entry)));
		UPDATE_MEMBER(user, user_entry);
		UPDATE_MEMBER(pass, pass_entry);

		config->auto_update = gtk_toggle_button_get_active
			(GTK_TOGGLE_BUTTON(auto_update_chkbtn));
		config->enable_searcher = gtk_toggle_button_get_active
			(GTK_TOGGLE_BUTTON(enable_chkbtn));
		UPDATE_MEMBER(exclude_list, exclude_entry);

		config->res_limit =
			strtol(gtk_entry_get_text(GTK_ENTRY(limit_entry)),
						  NULL, 10);
		if (config->res_limit < 0 || config->res_limit >= G_MAXINT)
			config->res_limit = 0;

		update_exclude_list(config);
		write_config(config);
		ext_send_msg("RELOAD DBCONFIG");
		ext_recv_msg(buf, sizeof(buf));
	}

#undef UPDATE_MEMBER

	gtk_widget_destroy(dialog);
	syl_plugin_inc_unlock();

	return (result == GTK_RESPONSE_OK);
}

static struct LicenseDialog {
	GtkWidget *dialog;
	GtkWidget *label;
	GtkWidget *hbox;
	gint auth_state;
} license_dialog;

static void register_clicked(GtkButton *button, gpointer data)
{
	GtkEntry *entry = GTK_ENTRY(data);
	gchar *key;

	key = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
	g_strchomp(key);
	if (key && *key && check_license_key_check_only("user", key) == 0) {
		if (check_license_key("user", key) == 0) {
			GList *children, *cur;
			GtkWidget *label;

			if (write_license_key(key) != 0) {
				g_warning("write_license_key failed");
				g_free(key);
				return;
			}

			if (license_dialog.auth_state < 0) {
				if (start_database() < 0) {
					g_warning("start database failed");
				}
			}
			license_dialog.auth_state = 0;

			gtk_label_set_text(GTK_LABEL(license_dialog.label), _("The following license is issued to this product:"));
			children = gtk_container_get_children(GTK_CONTAINER(license_dialog.hbox));
			for (cur = children; cur != NULL; cur = cur->next) {
				gtk_widget_destroy(GTK_WIDGET(cur->data));
			}
			g_list_free(children);			
			label = gtk_label_new(_("License key:"));
			gtk_box_pack_start(GTK_BOX(license_dialog.hbox), label, FALSE, FALSE, 0);
			label = gtk_label_new(key);
			gtk_label_set_selectable(GTK_LABEL(label), TRUE);
			gtk_box_pack_start(GTK_BOX(license_dialog.hbox), label, FALSE, FALSE, 0);
			gtk_widget_show_all(license_dialog.hbox);
		}
	} else {
		alert_dialog(_("Error"), _("Invalid license key."),
			     GTK_WINDOW(license_dialog.dialog));
	}

	g_free(key);
}

static void license_clicked(GtkButton *button, gpointer data)
{
	GtkWidget *dialog;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *reg_btn;
	gchar *key;
	GtkWindow *parent = GTK_WINDOW(data);
	gint auth_state = -1;
	AppConfig *config;

	config = searcher_get_config();
	if (!config->enable_searcher) {
		alert_dialog(_("Error"), _("Full-text search feature is disabled.\nPlease enable it and restart Sylpheed."), parent);
		return;
	}

	key = read_license_key();

	if (key && *key != '\0')
		auth_state = check_license_key_check_only("user", key);

	if (auth_state < -2) {
		alert_dialog(_("Error"), _("Cannot check the license key. Please restart Sylpheed."), parent);
		return;
	}

	dialog = gtk_dialog_new_with_buttons(_("License information"),
					     parent, 0,
					     GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
					     NULL);
	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, FALSE);
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);

	vbox = gtk_vbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox,
			   FALSE, FALSE, 0);

	license_dialog.auth_state = auth_state;

	if (auth_state == 0) {
		label = gtk_label_new(_("The following license is issued to this product:"));
		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
		license_dialog.label = label;

		hbox = gtk_hbox_new(FALSE, 4);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		license_dialog.hbox = hbox;

		label = gtk_label_new(_("License key:"));
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		label = gtk_label_new(key);
		gtk_label_set_selectable(GTK_LABEL(label), TRUE);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	} else {
		if (auth_state >= 1) {
			gchar buf[1024];

			g_snprintf(buf, sizeof(buf),
				   _("Trial license is issued to this product (%d day(s) left).\n\n"
				     "If you have received a license key,\n"
				     "enter it into the following entry and click 'Register'."), auth_state - 1);
			label = gtk_label_new(buf);
		} else if (auth_state == -2) {
			label = gtk_label_new(_("Trial license has been expired.\n\n"
						"If you have received a license key,\n"
						"enter it into the following entry and click 'Register'."));
		} else {
			label = gtk_label_new(_("This product is not licensed yet.\n\n"
						"If you have received a license key,\n"
						"enter it into the following entry and click 'Register'."));
		}
		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
		license_dialog.label = label;

		hbox = gtk_hbox_new(FALSE, 4);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		license_dialog.hbox = hbox;

		entry = gtk_entry_new();
		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
		reg_btn = gtk_button_new_with_mnemonic(_("_Register"));
		gtk_box_pack_start(GTK_BOX(hbox), reg_btn, FALSE, FALSE, 0);
		g_signal_connect(G_OBJECT(reg_btn), "clicked",
				 G_CALLBACK(register_clicked), entry);
	}

	license_dialog.dialog = dialog;

	gtk_widget_show_all(vbox);

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	license_dialog.dialog = NULL;
	license_dialog.label = NULL;
	license_dialog.hbox = NULL;
	license_dialog.auth_state = 0;
}
